#!/usr/bin/env node
'use strict';

/* eslint-disable no-use-before-define */

// Dependencies
var chalk = require('chalk');
var inquirer = require('inquirer');
// const yargs = require('yargs');
var netbank = require('..');
var moment = require('moment-timezone');
var Table = require('easy-table');
var debug = require('debug')('node-cba-netbank');

moment.tz.setDefault('Australia/Sydney');

var accounts = [];

var renderCurrency = function renderCurrency(amount) {
  return amount >= 0 ? chalk.green.bold('$' + amount) : chalk.red.bold('$' + amount);
};

function getAccountTitle(account) {
  var title = chalk.bold(account.name) + ' (' + chalk.red(account.bsb) + ' ' + chalk.red(account.account) + ')  =>  Balance: ' + renderCurrency(account.balance) + ' Available Funds: ' + renderCurrency(account.available);
  return title;
}

function fetchHistory(answer) {
  debug('fetchHistory(): accounts[' + accounts.length + ']');
  debug('[' + accounts.map(function (a) {
    return a.name;
  }) + ']');
  var account = accounts.find(function (v) {
    return answer.account.number === v.number;
  });
  if (account) {
    debug('fetchHistory(' + getAccountTitle(answer.account) + '): found account ' + getAccountTitle(account));
    //  download 6 months transactions
    console.log('Downloading 6 months history for ' + getAccountTitle(account) + ' ...');
    return netbank.getTransactions(account, netbank.toDateString(moment().subtract(6, 'months').valueOf())).then(showTransactions);
  }
  throw new Error('Quit.');
}

function showTransactions(resp) {
  console.log(Table.print(resp.transactions.map(function (t) {
    return {
      Time: chalk.italic.yellow(moment(t.timestamp).format('YYYY-MM-DD HH:mm')),
      Description: t.description.replace(/\n/g, ''),
      Amount: renderCurrency(t.amount)
    };
  })));
  console.log('Retrieved ' + resp.transactions.length + ' transactions.');
  return inquirer.prompt({
    type: 'list',
    name: 'account',
    message: 'Which account?',
    choices: accounts.map(function (a) {
      return { name: getAccountTitle(a), value: a };
    }).concat([chalk.red('<Quit>')])
  }).then(fetchHistory);
}

function login(credential) {
  console.log('Logging into the netbank as account ' + credential.username + ' ...');
  return netbank.login(credential).catch(function () {
    return inquirer.prompt({
      type: 'confirm',
      name: 'tryagain',
      message: 'Failed to logged in, try again?'
    }).then(function (answer) {
      if (answer.tryagain) {
        console.log();
        return main();
      }
      throw new Error('Quit');
    });
  }
  //  user select an account to download transaction, or quit.
  ).then(function (resp) {
    accounts = resp.accounts;
    return inquirer.prompt({
      type: 'list',
      name: 'account',
      message: 'Which account?',
      choices: resp.accounts.map(function (a) {
        return { name: getAccountTitle(a), value: a };
      }).concat([chalk.red('<Quit>')])
    });
  }).then(fetchHistory);
}

function main() {
  var credential = {
    username: process.env.NETBANK_USERNAME,
    password: process.env.NETBANK_PASSWORD
  };
  //  user input credential
  var questions = [];

  //  client number
  if (!credential.username) {
    questions.push({
      type: 'input',
      name: 'username',
      message: "What's your client number:",
      validate: function validate(input) {
        return (/^\d{1,8}$/.test(input)
        );
      }
    });
  }

  //  password
  if (!credential.password) {
    questions.push({
      type: 'password',
      name: 'password',
      message: "What's your password:",
      validate: function validate(input) {
        return (/^[\w\d-]{4,16}$/.test(input)
        );
      }
    });
  }

  if (questions.length > 0) {
    return inquirer.prompt(questions).then(login);
  }
  return login(credential);
}

console.table([{ name: 'foo' }, { name: 'bar' }]);

main().then(function () {
  return console.log('bye');
}).catch(debug);